@@ -174,7 +174,6 @@ PyString_AsString(PyObject *unicode)
 {
 	char	   *rv;
 	PyObject   *o = PyUnicode_AsEncodedString(unicode, GetDatabaseEncodingName(), NULL);
-
 	errorCheck();
 	rv = pstrdup(PyBytes_AsString(o));
 	Py_XDECREF(o);
@@ -186,12 +185,14 @@ PyString_AsStringAndSize(PyObject *obj, char **buffer, Py_ssize_t *length)
 {
 	PyObject   *o;
 	int			rv;
+	char *tempbuffer;
 
 	if (PyUnicode_Check(obj))
 	{
 		o = PyUnicode_AsEncodedString(obj, GetDatabaseEncodingName(), NULL);
 		errorCheck();
-		rv = PyBytes_AsStringAndSize(o, buffer, length);
+		rv = PyBytes_AsStringAndSize(o, &tempbuffer, length);
+		*buffer = pstrdup(tempbuffer);
 		Py_XDECREF(o);
 		return rv;
 	}
@@ -931,7 +932,7 @@ execute(ForeignScanState *node, ExplainState *es)
 				newqual->base.isArray = qual->isArray;
 				newqual->base.useOr = qual->useOr;
 				newqual->value = ExecEvalExpr(expr_state, econtext, &isNull, NULL);
-				newqual->base.typeoid = qual->typeoid;
+				newqual->base.typeoid = ((Param*) ((MulticornParamQual *) qual)->expr)->paramtype;
 				newqual->isnull = isNull;
 				break;
 			case T_Const:
